/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "fttypes.h"
#include "ftalloc.h"
#include "ftop.h"

#include <stdio.h>
#include <unistd.h>

/*
 * Simulate the following example:

txt=ftrobopy.ftrobopy('auto')

ml=txt.motor(1)
mr=txt.motor(2)
t1=txt.trailfollower(2)
t2=txt.trailfollower(3)

ml.setSpeed(512)
mr.setSpeed(512)
while True:
  if t1.state() < 1000 and t2.state() > 1000:
    ml.setSpeed(0)
  else:
    ml.setSpeed(512)
  if t2.state() < 1000 and t1.state() > 1000:
    mr.setSpeed(0)
  else:
    mr.setSpeed(512)
  time.sleep(0.02)

 */

int main(int argc, char **argv)
{
    (void)argc; (void)argv;
    void *txt = ftop_alloc_txt("auto", 65000);
    void *motorl;
    void *motorr;
    void *trail1;
    void *trail2;

    if (txt == NULL) {
        fprintf(stderr,"Can't conect 'auto'\n");
	return 1;
    }

    motorl = ftop_alloc_motor(txt, ftdev_m1); /* ml=txt.motor(1) */
    motorr = ftop_alloc_motor(txt, ftdev_m2); /* mr=txt.motor(2) */
    trail1 = ftop_alloc_trail_follower(txt, ftdev_i2); /* t1=txt.trailfollower(2) */
    trail2 = ftop_alloc_trail_follower(txt, ftdev_i3); /* t2=txt.trailfollower(3) */

    sleep(1);
    ftop_update_config(txt);

    /* Start motors */
    ftop_motor_start_speed(motorl, 512); /* ml.setSpeed(512) */
    ftop_motor_start_speed(motorr, 512); /* mr.setSpeed(512) */

    /* Process loop */
    while(1) {
        int t1s = ftop_get_state(trail1); /* t1.state() */
        int t2s = ftop_get_state(trail2); /* t2.state() */

        fprintf(stderr, "t1s %d t2s %d\n", t1s, t2s);

        if (t1s < 1000 && t2s > 1000) { /* if t1.state() < 1000 and t2.state() > 1000 */
            ftop_motor_start_speed(motorl, 0); /* ml.setSpeed(0) */
        } else {
            ftop_motor_start_speed(motorl, 512); /* ml.setSpeed(512) */
        }

        if (t2s < 1000 && t1s > 1000) { /* if t1.state() < 1000 and t2.state() > 1000 */
            ftop_motor_start_speed(motorr, 0); /* mr.setSpeed(0) */
        } else {
            ftop_motor_start_speed(motorr, 512); /* mr.setSpeed(512) */
        }

        usleep(1000000/50); /* time.sleep(0.02) */
    }

    return 0;
}

