/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file trlamp.c
 * @brief Simple FT API example for a remote lamp
 */

/*
# This test changes remote lamp brightnesss
# Lamp must be connected to remote O7 and GND (O8 also can be used as GND)

import time
import ft
ltxt2=ft.fttxt2()
l=ltxt2.getControllerInfoList()
txt2=ltxt2.getController(l[1])
l7=txt2.lamp(7)
txt2.update_config()
print("Remote lamp test")
i=0
while i <= 512:
    l7.pwm_set(i)
    i = i + 50
    time.sleep(1)

l7.pwm_set(0)
*/

#include "ftlock.h"
#include "ft.h"

#include <stdio.h>
#include <unistd.h>

/**
 * @brief Standard main procedure
 * @param [in] argc number of arguments
 * @param [in] argv arguments
 * @return Error code
 */
int main(int argc, char **argv)
{
    if (ftlock_open() < 0)
	return 1;

    if (ftlock_try_lock(argv[0]) < 0)
	return 1;

    void *ltxt = ftop_alloc_txt("auto", 65000);

    (void)argc; (void)argv;

    if (ltxt == NULL) {
        fprintf(stderr,"Can't conect %s\n", argv[1]);
        return 1;
    }

    void *txt = ftop_get_slave_by_order(ltxt, 1);

    if (txt == NULL) {
        fprintf(stderr,"Slave 1 is not available\n");
        return 1;
    }

    void *l7 = ftop_alloc_lamp(txt, ftdev_o7);

    if (l7 == NULL) {
    	fprintf(stderr,"Can't allocate O7 lamp\n");
    	return 1;
    }

    ftop_update_config(txt);

    ftlock_notify('R');

    fprintf(stderr,"Remote lamp test\n");

    int i;

    for (i = 0; i <= 512; i += 20) {
    	ftop_set_brightness(l7, i);
    	sleep(1);
    }

    ftop_set_brightness(l7, 0);

    return 0;
}
