/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file  HWServo.c
 * @brief Servo control procedures
 */

#include "HWServo.h"
#include "main.h"
#include "fttypes.h"

#include "cmsis_os.h"

extern TIM_HandleTypeDef htim2; /*!< TIM2 HAL handle */

#if TXT2_BOARD_REVISION == 0
extern TIM_HandleTypeDef htim4; /*!< TIM4 HAL handle */
extern TIM_HandleTypeDef htim5; /*!< TIM5 HAL handle */
#endif /* TXT2_BOARD_REVISION */

/*!< Initialization structure for servo motors PWM */
struct ft_servo_motors {
    TIM_HandleTypeDef* htim; 	/*!< pwm timer handle*/
    TIM_TypeDef*	   tim;     /*!< pwm timer address*/
    uint32_t 		   Channel; /*!< pwm channel*/
};

static const struct ft_servo_motors ftServoMotors[FT_MAX_SERVO] = {
#if TXT2_BOARD_REVISION == 0
		{ &htim4, TIM4, TIM_CHANNEL_3 },
		{ &htim5, TIM5, TIM_CHANNEL_2 },
		{ &htim2, TIM2, TIM_CHANNEL_3 }
#else
		{ &htim2, TIM2, TIM_CHANNEL_3 },
		{ &htim2, TIM2, TIM_CHANNEL_2 },
		{ &htim2, TIM2, TIM_CHANNEL_1 },
#endif /* TXT2_BOARD_REVISION */
};

/**
 * @brief S1-S3 servo initialization procedure
 */
void HWServo_Init()
{
#if TXT2_BOARD_REVISION == 1
    /* Servo enable */
	GPIO_InitTypeDef  GPIO_InitStruct;

	GPIO_InitStruct.Pin = SERVO_EN_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(SERVO_EN_GPIO_Port, &GPIO_InitStruct);
	HAL_GPIO_WritePin(SERVO_EN_GPIO_Port, SERVO_EN_Pin, GPIO_PIN_SET);

	GPIO_InitStruct.Pin = SERVO_OC_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
	HAL_GPIO_Init(SERVO_OC_GPIO_Port, &GPIO_InitStruct);
#endif

	for (int i = 0; i < FT_MAX_SERVO; i++) {
		HAL_TIM_PWM_Start(ftServoMotors[i].htim, ftServoMotors[i].Channel);
	}
}

/**
 * @brief Servo set duty procedure
 * @param n servo number 0-2
 * @param duty new duty value 0-512
 */
void HWServo_SetDuty(unsigned int n, uint16_t duty)
{
	if (n >= FT_MAX_SERVO)
		return;

	__HAL_TIM_SET_COMPARE(ftServoMotors[n].htim, ftServoMotors[n].Channel,
			SERVO_MIN_PULSE + (SERVO_MAX_PULSE - SERVO_MIN_PULSE) * duty / 512);
}
